@@ -1,30 +1,22 @@
-# Copyright (c) 2007, 2009-2010, 2013 LOGILAB S.A. (Paris, FRANCE) <contact@logilab.fr>
-# Copyright (c) 2014 Google, Inc.
-# Copyright (c) 2015-2018, 2020 Claudiu Popa <pcmanticore@gmail.com>
-# Copyright (c) 2015-2016 Ceridwen <ceridwenv@gmail.com>
-# Copyright (c) 2016 Derek Gustafson <degustaf@gmail.com>
-# Copyright (c) 2018 Bryce Guinta <bryce.paul.guinta@gmail.com>
-# Copyright (c) 2020-2021 hippo91 <guillaume.peillex@gmail.com>
-# Copyright (c) 2021 Daniël van Noord <13665637+DanielNoord@users.noreply.github.com>
-# Copyright (c) 2021 Pierre Sassoulas <pierre.sassoulas@gmail.com>
-# Copyright (c) 2021 Marc Mueller <30130371+cdce8p@users.noreply.github.com>
-# Copyright (c) 2021 Andrew Haigh <hello@nelf.in>
-
 # Licensed under the LGPL: https://www.gnu.org/licenses/old-licenses/lgpl-2.1.en.html
-# For details: https://github.com/PyCQA/astroid/blob/main/LICENSE
+# For details: https://github.com/pylint-dev/astroid/blob/main/LICENSE
+# Copyright (c) https://github.com/pylint-dev/astroid/blob/main/CONTRIBUTORS.txt
 
-"""this module contains exceptions used in the astroid library
-"""
-from typing import TYPE_CHECKING
+"""This module contains exceptions used in the astroid library."""
 
-from astroid import util
+from __future__ import annotations
+
+from collections.abc import Iterable, Iterator
+from typing import TYPE_CHECKING, Any
+
+from astroid.typing import InferenceResult, SuccessfulInferenceResult
 
 if TYPE_CHECKING:
-    from astroid import nodes
+    from astroid import arguments, bases, nodes, objects
+    from astroid.context import InferenceContext
 
 __all__ = (
     "AstroidBuildingError",
-    "AstroidBuildingException",
     "AstroidError",
     "AstroidImportError",
     "AstroidIndexError",
@@ -32,7 +24,6 @@ __all__ = (
     "AstroidTypeError",
     "AstroidValueError",
     "AttributeInferenceError",
-    "BinaryOperationError",
     "DuplicateBasesError",
     "InconsistentMroError",
     "InferenceError",
@@ -41,19 +32,19 @@ __all__ = (
     "NameInferenceError",
     "NoDefault",
     "NotFoundError",
-    "OperationError",
+    "ParentMissingError",
     "ResolveError",
+    "StatementMissing",
     "SuperArgumentTypeError",
     "SuperError",
     "TooManyLevelsError",
-    "UnaryOperationError",
     "UnresolvableName",
     "UseInferenceDefault",
 )
 
 
 class AstroidError(Exception):
-    """base exception class for all astroid related exceptions
+    """Base exception class for all astroid related exceptions.
 
     AstroidError and its subclasses are structured, intended to hold
     objects representing state when the exception is thrown.  Field
@@ -66,25 +57,41 @@ class AstroidError(Exception):
     arguments.
     """
 
-    def __init__(self, message="", **kws):
+    def __init__(self, message: str = "", **kws: Any) -> None:
         super().__init__(message)
         self.message = message
         for key, value in kws.items():
             setattr(self, key, value)
 
-    def __str__(self):
+    def __str__(self) -> str:
         return self.message.format(**vars(self))
 
 
 class AstroidBuildingError(AstroidError):
-    """exception class when we are unable to build an astroid representation
+    """Exception class when we are unable to build an astroid representation.
 
     Standard attributes:
         modname: Name of the module that AST construction failed for.
         error: Exception raised during construction.
     """
 
-    def __init__(self, message="Failed to import module {modname}.", **kws):
+    def __init__(
+        self,
+        message: str = "Failed to import module {modname}.",
+        modname: str | None = None,
+        error: Exception | None = None,
+        source: str | None = None,
+        path: str | None = None,
+        cls: type | None = None,
+        class_repr: str | None = None,
+        **kws: Any,
+    ) -> None:
+        self.modname = modname
+        self.error = error
+        self.source = source
+        self.path = path
+        self.cls = cls
+        self.class_repr = class_repr
         super().__init__(message, **kws)
 
 
@@ -100,34 +107,51 @@ class TooManyLevelsError(AstroidImportError):
         name: the name of the module on which the relative import was attempted.
     """
 
-    level = None
-    name = None
-
     def __init__(
         self,
-        message="Relative import with too many levels " "({level}) for module {name!r}",
-        **kws,
-    ):
+        message: str = "Relative import with too many levels "
+        "({level}) for module {name!r}",
+        level: int | None = None,
+        name: str | None = None,
+        **kws: Any,
+    ) -> None:
+        self.level = level
+        self.name = name
         super().__init__(message, **kws)
 
 
 class AstroidSyntaxError(AstroidBuildingError):
     """Exception class used when a module can't be parsed."""
 
+    def __init__(
+        self,
+        message: str,
+        modname: str | None,
+        error: Exception,
+        path: str | None,
+        source: str | None = None,
+    ) -> None:
+        super().__init__(message, modname, error, source, path)
+
 
 class NoDefault(AstroidError):
-    """raised by function's `default_value` method when an argument has
-    no default value
+    """Raised by function's `default_value` method when an argument has
+    no default value.
 
     Standard attributes:
         func: Function node.
         name: Name of argument without a default.
     """
 
-    func = None
-    name = None
-
-    def __init__(self, message="{func!r} has no default for {name!r}.", **kws):
+    def __init__(
+        self,
+        message: str = "{func!r} has no default for {name!r}.",
+        func: nodes.FunctionDef | None = None,
+        name: str | None = None,
+        **kws: Any,
+    ) -> None:
+        self.func = func
+        self.name = name
         super().__init__(message, **kws)
 
 
@@ -140,7 +164,11 @@ class ResolveError(AstroidError):
         context: InferenceContext object.
     """
 
-    context = None
+    def __init__(
+        self, message: str = "", context: InferenceContext | None = None, **kws: Any
+    ) -> None:
+        self.context = context
+        super().__init__(message, **kws)
 
 
 class MroError(ResolveError):
@@ -152,10 +180,20 @@ class MroError(ResolveError):
         context: InferenceContext object.
     """
 
-    mros = ()
-    cls = None
+    def __init__(
+        self,
+        message: str,
+        mros: Iterable[Iterable[nodes.ClassDef]],
+        cls: nodes.ClassDef,
+        context: InferenceContext | None = None,
+        **kws: Any,
+    ) -> None:
+        self.mros = mros
+        self.cls = cls
+        self.context = context
+        super().__init__(message, **kws)
 
-    def __str__(self):
+    def __str__(self) -> str:
         mro_names = ", ".join(f"({', '.join(b.name for b in m)})" for m in self.mros)
         return self.message.format(mros=mro_names, cls=self.cls)
 
@@ -176,24 +214,61 @@ class SuperError(ResolveError):
         context: InferenceContext object.
     """
 
-    super_ = None
+    def __init__(self, message: str, super_: objects.Super, **kws: Any) -> None:
+        self.super_ = super_
+        super().__init__(message, **kws)
 
-    def __str__(self):
+    def __str__(self) -> str:
         return self.message.format(**vars(self.super_))
 
 
-class InferenceError(ResolveError):
-    """raised when we are unable to infer a node
+class InferenceError(ResolveError):  # pylint: disable=too-many-instance-attributes
+    """Raised when we are unable to infer a node.
 
     Standard attributes:
         node: The node inference was called on.
         context: InferenceContext object.
     """
 
-    node = None
-    context = None
-
-    def __init__(self, message="Inference failed for {node!r}.", **kws):
+    def __init__(  # pylint: disable=too-many-arguments
+        self,
+        message: str = "Inference failed for {node!r}.",
+        node: InferenceResult | None = None,
+        context: InferenceContext | None = None,
+        target: InferenceResult | None = None,
+        targets: InferenceResult | None = None,
+        attribute: str | None = None,
+        unknown: InferenceResult | None = None,
+        assign_path: list[int] | None = None,
+        caller: SuccessfulInferenceResult | None = None,
+        stmts: Iterator[InferenceResult] | None = None,
+        frame: InferenceResult | None = None,
+        call_site: arguments.CallSite | None = None,
+        func: InferenceResult | None = None,
+        arg: str | None = None,
+        positional_arguments: list | None = None,
+        unpacked_args: list | None = None,
+        keyword_arguments: dict | None = None,
+        unpacked_kwargs: dict | None = None,
+        **kws: Any,
+    ) -> None:
+        self.node = node
+        self.context = context
+        self.target = target
+        self.targets = targets
+        self.attribute = attribute
+        self.unknown = unknown
+        self.assign_path = assign_path
+        self.caller = caller
+        self.stmts = stmts
+        self.frame = frame
+        self.call_site = call_site
+        self.func = func
+        self.arg = arg
+        self.positional_arguments = positional_arguments
+        self.unpacked_args = unpacked_args
+        self.keyword_arguments = keyword_arguments
+        self.unpacked_kwargs = unpacked_kwargs
         super().__init__(message, **kws)
 
 
@@ -208,10 +283,17 @@ class NameInferenceError(InferenceError):
         context: InferenceContext object.
     """
 
-    name = None
-    scope = None
-
-    def __init__(self, message="{name!r} not found in {scope!r}.", **kws):
+    def __init__(
+        self,
+        message: str = "{name!r} not found in {scope!r}.",
+        name: str | None = None,
+        scope: nodes.LocalsDictNodeNG | None = None,
+        context: InferenceContext | None = None,
+        **kws: Any,
+    ) -> None:
+        self.name = name
+        self.scope = scope
+        self.context = context
         super().__init__(message, **kws)
 
 
@@ -224,78 +306,111 @@ class AttributeInferenceError(ResolveError):
         context: InferenceContext object.
     """
 
-    target = None
-    attribute = None
-
-    def __init__(self, message="{attribute!r} not found on {target!r}.", **kws):
+    def __init__(
+        self,
+        message: str = "{attribute!r} not found on {target!r}.",
+        attribute: str = "",
+        target: nodes.NodeNG | bases.BaseInstance | None = None,
+        context: InferenceContext | None = None,
+        mros: list[nodes.ClassDef] | None = None,
+        super_: nodes.ClassDef | None = None,
+        cls: nodes.ClassDef | None = None,
+        **kws: Any,
+    ) -> None:
+        self.attribute = attribute
+        self.target = target
+        self.context = context
+        self.mros = mros
+        self.super_ = super_
+        self.cls = cls
         super().__init__(message, **kws)
 
 
 class UseInferenceDefault(Exception):
-    """exception to be raised in custom inference function to indicate that it
-    should go back to the default behaviour
+    """Exception to be raised in custom inference function to indicate that it
+    should go back to the default behaviour.
     """
 
 
 class _NonDeducibleTypeHierarchy(Exception):
-    """Raised when is_subtype / is_supertype can't deduce the relation between two types."""
+    """Raised when is_subtype / is_supertype can't deduce the relation between two
+    types.
+    """
 
 
 class AstroidIndexError(AstroidError):
     """Raised when an Indexable / Mapping does not have an index / key."""
 
+    def __init__(
+        self,
+        message: str = "",
+        node: nodes.NodeNG | bases.Instance | None = None,
+        index: nodes.Subscript | None = None,
+        context: InferenceContext | None = None,
+        **kws: Any,
+    ) -> None:
+        self.node = node
+        self.index = index
+        self.context = context
+        super().__init__(message, **kws)
+
 
 class AstroidTypeError(AstroidError):
     """Raised when a TypeError would be expected in Python code."""
 
+    def __init__(
+        self,
+        message: str = "",
+        node: nodes.NodeNG | bases.Instance | None = None,
+        index: nodes.Subscript | None = None,
+        context: InferenceContext | None = None,
+        **kws: Any,
+    ) -> None:
+        self.node = node
+        self.index = index
+        self.context = context
+        super().__init__(message, **kws)
+
 
 class AstroidValueError(AstroidError):
     """Raised when a ValueError would be expected in Python code."""
 
 
 class InferenceOverwriteError(AstroidError):
-    """Raised when an inference tip is overwritten
+    """Raised when an inference tip is overwritten.
 
     Currently only used for debugging.
     """
 
 
 class ParentMissingError(AstroidError):
-    """Raised when a node which is expected to have a parent attribute is missing one
+    """Raised when a node which is expected to have a parent attribute is missing one.
 
     Standard attributes:
         target: The node for which the parent lookup failed.
     """
 
-    def __init__(self, target: "nodes.NodeNG") -> None:
+    def __init__(self, target: nodes.NodeNG) -> None:
         self.target = target
         super().__init__(message=f"Parent not found on {target!r}.")
 
 
 class StatementMissing(ParentMissingError):
-    """Raised when a call to node.statement() does not return a node. This is because
-    a node in the chain does not have a parent attribute and therefore does not
-    return a node for statement().
+    """Raised when a call to node.statement() does not return a node.
+
+    This is because a node in the chain does not have a parent attribute
+    and therefore does not return a node for statement().
 
     Standard attributes:
         target: The node for which the parent lookup failed.
     """
 
-    def __init__(self, target: "nodes.NodeNG") -> None:
-        # pylint: disable-next=bad-super-call
-        # https://github.com/PyCQA/pylint/issues/2903
-        # https://github.com/PyCQA/astroid/pull/1217#discussion_r744149027
+    def __init__(self, target: nodes.NodeNG) -> None:
         super(ParentMissingError, self).__init__(
             message=f"Statement not found on {target!r}"
         )
 
 
-# Backwards-compatibility aliases
-OperationError = util.BadOperationMessage
-UnaryOperationError = util.BadUnaryOperationMessage
-BinaryOperationError = util.BadBinaryOperationMessage
-
 SuperArgumentTypeError = SuperError
 UnresolvableName = NameInferenceError
 NotFoundError = AttributeInferenceError
-AstroidBuildingException = AstroidBuildingError
